/****************************************************************
**
**
****************************************************************/

#include "lwin.h"
#include "conf.h"

//----------

clwin::clwin( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
 int	 c; 
 QString s;

 initVars();

 cont = new QWidget( this );
#ifdef USEKDE
 datadir = app->kdedir()+TDATADS;
#else
 datadir = XDATPATH;
#endif 

 bp.load(datadir+"/bg1.xpm");
 
 for (c=0;c<B_MAX;c++)
  pcont[c].load((datadir+"/")+icnam[c]);

 createWidgets();
 setCaption("Tamaga");
 placeWidgets();

 initScheduler(); // must come before loadtama
 if (!loadTama())
  resetTama();

 startTama();
 startScheduler();
}

clwin::~clwin( void )
{
 saveTama();
}

void clwin::createMenu( void )
{
#ifdef USEKDE
 menu = new KMenuBar(cont,"Tamaga");
 menu->enableMoving(FALSE);
#else
 menu = new QMenuBar(cont,"Tamaga");
#endif
 fileMenu = new QPopupMenu;
 fileMenu->insertItem("&Configure...",this,SLOT(configMenu()),ALT+Key_C);
 fileMenu->insertItem("&Hide menubar", this, SLOT(hideMenu()),ALT+Key_M);
 fileMenu->insertSeparator();
 fileMenu->insertItem("&Quit",qApp,SLOT(quit()),ALT+Key_Q );
 
 helpMenu = new QPopupMenu;
 helpMenu->insertItem( "&Contents", this, SLOT(launchHelp()), ALT+Key_H);
 helpMenu->insertSeparator();
 helpMenu->insertItem( "&About", this, SLOT(aboutClicked())); 
 
 menu->insertItem("&File",fileMenu);
 menu->insertSeparator();
 menu->insertItem("&Help",helpMenu);

 pmenu = new QPopupMenu;
 pmenu->insertItem("&Configure...",this,SLOT(configMenu()),ALT+Key_C);
 pmenu->insertItem( "&Show menubar", this, SLOT(hideMenu()), ALT+Key_M );
}

void clwin::createWidgets( void )
{
 createMenu();
 createButt();
 scr = new tlcd( cont, "lcdmain" );
 cont->setBackgroundPixmap(bp);
 connect( scr, SIGNAL(endState()), SLOT(stateEnded()) );
 connect( scr, SIGNAL(clickedX( QPoint )), SLOT(popmenu( QPoint )) );
}
 
void clwin::placeWidgets( void )
{
 int ix=0,iy=0,c;

 scr->disconnect( this, SLOT(popmenu( QPoint )) );

 if (config->menuvis) {
  iy += menu->height();
  menu->show();
 } else {
  menu->show();
  menu->hide();

  connect( scr, SIGNAL(clickedX( QPoint )), SLOT(popmenu( QPoint )) );
 }

 for (c=0;c<8;c++)
  bcont[c]->move( 5+35*(c%4), iy+5+int(c/4)*105 ); 

 scr->setGeometry(5,35+iy,133,70);

 ix=147;
 menu->resize( ix, menu->height());
 iy+=142;

 cont->setFixedSize(ix,iy);
 setFixedSize(ix,iy); 
 resize(ix,iy);
}

void clwin::configMenu( )
{
 int fl=0;
 app_pause=1;
 
 config->prep();
 if ( config->exec() ) {
  scr->changeSet();
  if (config->dbgreq==1) scr->setPhase(++st.phase);
  if (config->dbgreq==2) { scr->setDebug(1); setButt(BS_DEBUG); 
                           app_pause=1; fl=1; }
  config->dbgreq=0;

  if (config->resetreq) {
   config->resetreq=0;
   resetTama();
   startTama();
   initScheduler();
  }
 }
 
 if (!fl)
  app_pause=0;
}

void clwin::hideMenu()
{
 config->menuvis=!config->menuvis;
 placeWidgets();
}

void clwin::aboutClicked()
{
 app_pause=1;
#ifdef USEKDE
 KMsgBox::message(0,"About",ABOUT_MSG,KMsgBox::INFORMATION,"OK");
#else
 QMessageBox::message("About",ABOUT_MSG,"OK");
#endif;
 app_pause=0;
}

void clwin::launchHelp()
{
 app_pause=1;
#ifdef USEKDE
 app->invokeHTMLHelp("ktamaga/ktamaga.html", "");
#else
 QMessageBox::message("Error!","For help, read the included HTML files.","OK"); 
#endif
 app_pause=0;
}

QPushButton *clwin::makeButton( int x, int y, int w, int h, const char *n,
 QPixmap *pix=NULL, char *tip=NULL )
{
   QPushButton *pb = new QPushButton( n, cont );
   pb->setGeometry( x, y, w, h );
   pb->setAutoRepeat(FALSE);
   if (pix) pb->setPixmap(*pix);
   if (tip) QToolTip::add( pb, tip );
   return pb;
} 

void clwin::changeCButton( int ind, QPixmap *pix=NULL, char *tip=NULL )
{
 QPushButton *pb=bcont[ind];
 if (pix) pb->setPixmap(*pix);
 if (tip) QToolTip::add( pb, tip );
} 

void clwin::setAttButt( int on )
{
 bcont[7]->setPixmap( on ? pcont[B_ATTLIGHT] : pcont[B_ATTNORM] );
}

void clwin::createButt( void )
{
 int c;

 for (c=0;c<8;c++)
  bcont[c] = makeButton( 5+35*(c%4), 0+5+int(c/4)*105, 30, 26, "Cntlx" );
}

void clwin::setButt( int st )
{
 printf("SetButt [%d]\n",st);
 int c;
 if (curst==0) {
 } else {
  for (c=0;c<8;c++)
   bcont[c]->disconnect(this);
 }

 curst=st;
 switch (st) {
  case BS_NORMAL:
		for (c=0;c<8;c++)
    		 changeCButton( c, &pcont[c], ictip[c] );
		if (ss.ars)
		 setAttButt(1);

	 	connect( bcont[0], SIGNAL(clicked()), SLOT(feedClicked()) );
 		connect( bcont[1], SIGNAL(clicked()), SLOT(lightsClicked()) );
 		connect( bcont[2], SIGNAL(clicked()), SLOT(gameClicked()) );
 		connect( bcont[3], SIGNAL(clicked()), SLOT(medicalClicked()) );
 		connect( bcont[4], SIGNAL(clicked()), SLOT(toiletClicked()) );
 		connect( bcont[5], SIGNAL(clicked()), SLOT(statusClicked()) );
 		connect( bcont[6], SIGNAL(clicked()), SLOT(teachClicked()) );
 		connect( bcont[7], SIGNAL(clicked()), SLOT(attentionClicked()) );
	       break;

  case BS_DEBUG:
		changeCButton( 0, &pcont[B_CANCEL], ictip[B_CANCEL]);
		for (c=0;c<7;c++)
    		 changeCButton( 1+c, &pcont[B_DEBUGSET+c], ictip[B_DEBUGSET+c] );

	 	connect( bcont[0], SIGNAL(clicked()), SLOT(dbgCancel()) );
 		connect( bcont[1], SIGNAL(clicked()), SLOT(dbgWrite()) );
		connect( bcont[2], SIGNAL(clicked()), SLOT(dbgPrev()) );
 		connect( bcont[3], SIGNAL(clicked()), SLOT(dbgNext()) );
 		connect( bcont[4], SIGNAL(clicked()), SLOT(dbgLeft()) );
 		connect( bcont[5], SIGNAL(clicked()), SLOT(dbgUp()) );
 		connect( bcont[6], SIGNAL(clicked()), SLOT(dbgDown()) );
 		connect( bcont[7], SIGNAL(clicked()), SLOT(dbgRight()) );
	       break;
 
  case BS_PLAY:	
    		changeCButton( 0, &pcont[B_CANCEL], ictip[B_CANCEL] );
    		changeCButton( 1, &pcont[B_TLEFT], ictip[B_TLEFT] );
    		changeCButton( 2, &pcont[B_TRIGHT], ictip[B_TRIGHT] );
		for (c=0;c<5;c++)
		 changeCButton( 3+c, &pcont[B_NULL], ictip[B_NULL] );

	 	connect( bcont[0], SIGNAL(clicked()), SLOT(gameCancel()) );
 		connect( bcont[1], SIGNAL(clicked()), SLOT(gameLeft()) );
 		connect( bcont[2], SIGNAL(clicked()), SLOT(gameRight()) );

	       break; 
  case BS_FEED:
		for (c=0;c<4;c++)
    		 changeCButton( c, &pcont[B_EATSET+c], ictip[B_EATSET+c] );
		for (c=0;c<4;c++)
		 changeCButton( 4+c, &pcont[B_NULL], ictip[B_NULL] );

	 	connect( bcont[0], SIGNAL(clicked()), SLOT(feedCancel()) );
 		connect( bcont[1], SIGNAL(clicked()), SLOT(feedMeal()) );
 		connect( bcont[2], SIGNAL(clicked()), SLOT(feedSnack()) );
               break;
 }
}


void clwin::stateEnded( void )
{
 printf("StateEnd [%d]\n",streq);
 if (streq) {
  scr->setState(streq);
  streq=0;
 } else {
  if (streq2) {
   scr->setState(streq2);
   streq2=0;
  }
 }
}

void clwin::needAttention( void )
{
 // if /dev/dsp exists/can be opened, then this should emit a sound... etc.
 if (!config->sound) return;
 
 if (ss.ars&AR_HATCH) {
  if (ctime%2==0)
   app->beep();
  return;
 } 

 if (ss.attn==5 || ss.attn==10 || ss.attn==20 || ss.attn==40)
  app->beep();
}

int clwin::loadTama( void )
{
 QFile	f(saveName);
 if (f.open(IO_Raw|IO_ReadOnly)==FALSE) 
  return 0;
 f.readBlock((char *)&st,sizeof(st));
 f.readBlock((char *)&ss,sizeof(ss));
 ctime=ss.ctimetmp;
 f.close();
 return 1;
}

int clwin::saveTama( void )
{
 QFile	f(saveName);

 printf("[Autosave %s]\n",saveName.data());
 if (f.open(IO_Raw|IO_WriteOnly|IO_Truncate)==FALSE) 
  { printf("[!Save error!!!]\n"); return 0; }
 ss.ctimetmp=ctime;
 f.writeBlock((char *)&st,sizeof(st));
 f.writeBlock((char *)&ss,sizeof(ss));
 f.close();
 return 1;
}

void clwin::resetTama( void )
{
 st.sleeping=0;
 st.lights=1;
 st.ill=0;
 st.hungry=2;
 st.happy=2;
 st.poop=0;
 st.ill=0;
 st.phase=TS_TAMAGOTCHI;
 st.bad=0;
 st.miscare=0;
 st.invdispl=0;
 st.fedplg=0;
 st.nodispl=0;

 st.age=0;
 st.wgt=5;
 st.discl=0; 
}

void clwin::startTama( void )
{
 streq=0;
 streq2=0;

 setButt(BS_NORMAL);
 scr->setPoop(st.poop);
 scr->setIll(st.ill);
 scr->setPhase(st.phase); 
 scr->setSNorm(AN_NORML);
 scr->setNormal();
}

void clwin::initVars( void )
{
 curst=0;

 saveName=getenv("HOME");
 CHECK_PTR(saveName);
 saveName+=TAMDATFILE;

 config = new conf;
 config->load();
}

int clwin::randx( int from, int to )
{
 return ( from  + (int)(float(to)*rand()/RAND_MAX) );
}   

float clwin::randf( float from, float to )
{
 return ( from  + (to*rand()/RAND_MAX) );
}   


void clwin::popmenu( QPoint p )
{
 pmenu->popup(p);
}
